;nyquist plug-in
;version 1
;type process
;categories "http://lv2plug.in/ns/lv2core/#DelayPlugin"
;name "Stereo Echo"
;action "Processing..."
;info "Stereo Echo by Steve Daulton (http://easyspacepro.com)\nReleased under GPL v2.\n"

;; StereoEcho.ny by Steve Daulton September 2010.
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
;;
;; For information about writing and modifying Nyquist plug-ins:
;; http://wiki.audacityteam.org/wiki/Nyquist_Plug-ins_Reference

;control left "Left Delay" real "seconds" 0.1 0 2.5
;control right "Right Delay" real "seconds" 0.1 0 2.5
;control fb "Feedback" real "%" 50 0 100
;control mix "Effect Mix" real "%" 30 0 100

(setq left (max 0 left))                  ; delay cannot be negative
(setq right (max 0 right))
(setq fb (max (min (/ fb 100.0) 1) 0))    ; restrict range 0 to 1
(setq mix (max (min (/ mix 100.0) 1) 0))  ; restrict range 0 to 1

;;; Convert mix range to dB curve
;;; Higher (optional) dbrange increases curve
(defun dbcurve (linear &optional (dbrange 24))
  (let ((dbrange (* -1 dbrange)))
    (* (+ 1 (db-to-linear dbrange))
       (- 1 (db-to-linear (* dbrange linear))))))

(defun process (sig delay fb mix)  
  (sim                                    ; mix (add) dry and wet components
    (scale (dbcurve (- 1 mix)) sig)       ; dry part
    (scale (dbcurve mix)                  ; wet part
           (feedback-delay sig delay fb))))

(cond
  ; left delay is longer than mono sound 
  ((and (soundp s)(> left (get-duration 1)))
    (format nil "Delayed mono sound is beyond the end of the selection.~%~
    Reduce the value of 'Left Delay' or select more audio."))
  ; left delay is longer than stereo sound
  ((> left (get-duration 1))
    (format nil "Delayed left channel is beyond the end of the selection.~%~
    Reduce the value of 'Left Delay' or select more audio."))
  ((> right (get-duration 1))
    (format nil "Delayed right channel is beyond the end of the selection.~%~
    Reduce the value of 'Right Delay' or select more audio."))
  (T                                    ; process audio
    (if (arrayp s)                      ; process stereo track
      (vector (process (aref s 0) left fb mix)
              (process (aref s 1) right fb mix))
      (process s left fb mix))))        ; process mono track


